##
## test_jambtables.py
## Login : <uli@pu.smp.net>
## Started on  Thu Jun 24 08:08:12 2010 Uli Fouquet
## $Id$
## 
## Copyright (C) 2010 Uli Fouquet
## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.
## 
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
## 
## You should have received a copy of the GNU General Public License
## along with this program; if not, write to the Free Software
## Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
##
"""
Tests for JAMB tables.

"""
import logging
import os
import shutil
import tempfile
import unittest

import grok

from cStringIO import StringIO
from datetime import datetime

from zope.interface.verify import verifyClass, verifyObject
from zope.site import LocalSiteManager
from zope.site.hooks import setSite, clearSite
from zope.site.testing import createSiteManager

from waeup.sirp.app import University
from waeup.sirp.datacenter import DataCenter
from waeup.sirp.testing import WAeUPSIRPUnitTestLayer
from waeup.sirp.applicants.jambtables import (
    JAMBDataPlugin, JAMBDataRoot, get_jambdata,
    )
from waeup.sirp.applicants.jambtables.interfaces import (
    IJAMBDataRoot,
    )
from waeup.sirp.applicants import Applicant
from waeup.sirp.applicants.container import ApplicantsContainer


class FakeSite(dict):
    pass

class JAMBTableTestCaseBase(unittest.TestCase):

    def setUp(self):
        self.tempdir = tempfile.mkdtemp()
        self.datafile = os.path.join(
            self.tempdir, 'jambsample.csv'
            )
        open(self.datafile, 'wb').write(
            'reg_no,fst_sit_fname,lastname,firstname,middlenames,sex,date_of_birth,jamb_state,jamb_lga,course1,screening_date,screening_venue,entry_session,screening_type,ignore_this_col\n' + 
            '91100546DD,ISUOSUO MOSES ODOMERO,ISUOSUO,MOSES,ODOMERO,M,25/5/1982,DEL,ISO-S,BSCPOL,2009/09/25 09:00:00 GMT+1,REPRINT SLIP AS FROM WED 23/09/2009,9,pde,blah\n' +
            '91111834CC,DURUIHEOMA AUGUSTINA ADANNA,DURUIHEOMA,AUGUSTINA,ADANNA,F,15/4/1986,IMO,MBAIT,BSCPOL,2009/09/25 09:00:00 GMT+1,REPRINT SLIP AS FROM WED 23/09/2009,9,pde,blah\n' +
            '91109351AC,ARISERE EBIKEBUNA COMFORT,ARISERE,EBIKEBUNA,COMFORT,F,6/1/1984,EDO,OV-SW,BSCPOL,2009/09/25 09:00:00 GMT+1,REPRINT SLIP AS FROM WED 23/09/2009,9,pde,blah\n'
            )
        setSite()
        return

    def tearDown(self):
        shutil.rmtree(self.tempdir)
        return

class JAMBDataRootTestCase(unittest.TestCase):

    def test_interfaces(self):
        # Make sure the correct interfaces are implemented.
        self.assertTrue(
            verifyClass(IJAMBDataRoot, JAMBDataRoot)
            )
        self.assertTrue(
            verifyObject(IJAMBDataRoot, JAMBDataRoot())
            )
        return

class JAMBDataPluginTestCase(unittest.TestCase):
    def create_logger(self):
        # create a logger suitable for local tests.
        test_logger = logging.getLogger('waeup.sirp.jambdata.testlogger')
        log = StringIO()
        handler = logging.StreamHandler(log)
        handler.setLevel(logging.DEBUG)
        test_logger.addHandler(handler)
        test_logger.setLevel(logging.DEBUG)
        self.logger = test_logger
        self.log = log
        self.handler = handler
        return self.logger

    def remove_logger(self):
        del self.handler
        del self.logger
        del self.log
        pass

    def get_log(self):
        self.log.seek(0)
        return self.log.read()

    def setUp(self):
        self.create_logger()
        return

    def tearDown(self):
        self.remove_logger()
        return

    # Real tests start here...
    def test_pluginsetup(self):
        # Make sure we can add ApplicantsRoot to sites.
        site = FakeSite()
        plugin = JAMBDataPlugin()
        plugin.setup(site, 'blah', self.logger)
        self.assertTrue('jambdata' in site.keys())
        log = self.get_log()
        self.assertTrue('JAMBDataPlugin: Installed JAMB data root.' in log)
        return

    def test_update_new(self):
        # Run update on a site without applicants root.
        site = FakeSite()
        plugin = JAMBDataPlugin()
        plugin.update(site, 'blah', self.logger)
        self.assertTrue('jambdata' in site.keys())
        log = self.get_log()
        self.assertTrue('Updating site at <Unnamed Site>' in log)
        self.assertTrue('Installed JAMB data root.' in log)
        return

    def test_update_outdated(self):
        # Run update on a site with outdated applicants root.
        site = FakeSite()
        root = object() # # This is not a proper applicants root
        site['jambdata'] = root
        plugin = JAMBDataPlugin()
        plugin.update(site, 'blah', self.logger)
        self.assertTrue(site['jambdata'] is not root)
        self.assertTrue(isinstance(site['jambdata'], JAMBDataRoot))
        log = self.get_log()
        self.assertTrue('Outdated JAMB data folder detected' in log)
        self.assertTrue('Updating site at <Unnamed Site>' in log)
        self.assertTrue('Installed JAMB data root.' in log)
        return

    def test_update_uptodate(self):
        # Run update on a site with proper applicants root.
        site = FakeSite()
        root = JAMBDataRoot()
        site['jambdata'] = root
        plugin = JAMBDataPlugin()
        plugin.update(site, 'blah', self.logger)
        self.assertTrue(site['jambdata'] is root)
        log = self.get_log()
        self.assertTrue('Updating site at <Unnamed Site>' in log)
        self.assertTrue('Nothing to do' in log)
        return

    def test_update_log(self):
        # Check that sitename is used in log messages on updates.
        site = FakeSite()
        site.__name__ = 'my_site'
        plugin = JAMBDataPlugin()
        plugin.update(site, 'blah', self.logger)
        log = self.get_log()
        self.assertTrue('Updating site at my_site.' in log)
        return

class GetJAMBDataTestCase(unittest.TestCase):

    layer = WAeUPSIRPUnitTestLayer
    
    def setUp(self):
        self.app = University()
        # Fill up some JAMB data...
        self.appl1 = Applicant()
        self.appl2 = Applicant()
        self.app['jambdata']['91100546DD'] = self.appl1
        self.app['jambdata']['91100546DE'] = self.appl2
        createSiteManager(self.app, setsite=True)
        return

    def tearDown(self):
        clearSite()
        return

    def test_contained(self):
        self.assertEqual(get_jambdata('91100546DD'), self.appl1)
        return

    def test_not_contained(self):
        self.assertEqual(get_jambdata('not-existing'), None)
        return

def test_suite():
    suite = unittest.TestSuite()
    for testcase in [
        JAMBDataRootTestCase, JAMBDataPluginTestCase, GetJAMBDataTestCase,
        ]:
        suite.addTest(unittest.TestLoader().loadTestsFromTestCase(
                testcase
                )
        )
    return suite
